
/*
Logic for new softDelete hook
Field name deletedAt is initialized to -1; set to Date.now() on REMOVE.

const SKIP = require('@feathersjs/feathers').SKIP;

GET
  // Some service calls require we make a probing GET to see if the record is logically deleted.
  // The following prevents any hooks from running when the probing GET is done.
  // Just the GET on the DB is be done.
  if (params.$disableAllHooks) return SKIP

GET, UPDATE, PATCH with id, REMOVE with id
  Make service call with GET (id, { provider: undefined, $disableAllHooks: true })
    (need to add default ...authProps and developer can customize this call)
    if it throws, softDelete throws with that same error
    if (deletedAt exists && deletedAt > -1) softDelete throws with "record not found"
  return context so processing continues normally & called method is performed

PATCH without id, REMOVE without id
  add { query: { deletedAt: { $eq: -1 } } } to context.params
  return context so processing continues normally

CREATE
  add { deletedAt: -1 } to context.data

FIND
  add { query: { deletedAt: { $ne: -1 } } }
 */
const SKIP = require('@feathersjs/feathers').SKIP;
const errors = require('@feathersjs/errors');
const checkContext = require('./check-context');
const { callingParams } = require('./callingParams');

module.exports = function (customGetCall, customRemoveCall, deletedAt = 'deletedAt') {
  return async function (context) {
    const service = context.service;
    const method = context.method;
    const hasId = context.id !== undefined && context.id !== null;
    checkContext(context, 'before', null, 'softDelete2');

    // This hook makes a probing get to see if the record is marked deleted.
    // It also converts a remove call to a patch.
    // Skip all hooks, including auth ones, on that probe. The user record is still returned.
    if (context.params.$disableSoftDelete2 && (method === 'get' || method === 'patch')) {
      return SKIP;
    }

    switch (method) {
      case 'find':
        context.params.query = Object.assign({}, { [deletedAt]: { $eq: -1 } }, context.params.query || {});
        return context;
      case 'create':
        context.params.data = Object.assign({}, context.params.data || {}, { [deletedAt]: -1 });
        return context;
      case 'get': // fall through
      case 'update':
        // what if deletedAt is being modified? do we care?
        await getActiveRec();
        return context;
      case 'patch':
        // what if deletedAt is being modified? do we care?
        if (hasId) {
          await getActiveRec();
          return context;
        }

        context.params.query = Object.assign({}, { [deletedAt]: { $eq: -1 } }, context.params.query || {});
        return context;
      case 'remove':
        throw new errors.BadRequest('Remove is not allowed for a service using sodtDelete2.', { id: context.id });

        if (hasId) {
          await getActiveRec();
          context.result = service.patch(
            context.id, { [deletedAt]: Date.now() }, { provider: undefined, $disableSoftDelete2: true }
          );
          return context;
        }

        context.params.query = Object.assign({}, { [deletedAt]: { $eq: -1 } }, context.params.query || {});
        context.result = service.patch(
          context.id, { [deletedAt]: Date.now() }, { provider: undefined, $disableSoftDelete2: true }
        );
        return context;
    }

    async function getActiveRec() {
      const record = service.get(context.id, { provider: undefined, $disableSoftDelete2: true });
      if (record[deletedAt] > -1) {
        throw new errors.NotFound('Record not found, is logically deleted.', { id: context.id });
      }

      return record;
    }
  };
};

async function softDeleteRemove(service, id, params) {
  return service.patch(id, { [deletedAt]: Date.now() }, params)
}